#!/usr/bin/env python
# -*- Coding: utf-8 -*-

import web
import time
import user
import todolib
import todotool
import config

class list:
	def GET(self, message=None):
		"""This method lists the TODOs."""

		# Authentification
		if todolib.valid_cookie() == False:
			user.validate().GET(redirect=web.ctx.path, message='Please log in...')
			return

		# Selecting the todos from the database


		if web.ctx.path.split('/')[1] == 'my':
			session = todolib.getSession()
			userid = str(session['userid'])
			undone = web.select('todos', order="deadline", where="done=0 and deleted=0 and who=" + userid)
			done = web.select('todos', order="deadline", where="done=1 and deleted=0 and who=" + userid)
		else:
			undone = web.select('todos', order="deadline", where="done=0 and deleted=0")
			done = web.select('todos', order="deadline", where="done=1 and deleted=0")


		users = web.select('users')

		# Dictionary to map a userid to a user
		usertab = {}
		for i in users:
				usertab[i['id']] = web.websafe(i['name'])

		web.render('todolist.html')




class add:
	def GET(self):
		"""This method show a form to add a new TODO."""
		
		# Authentification
		if todolib.valid_cookie() == False:
			user.validate().GET(redirect=web.ctx.path, message='Please log in...')
			return


		# Database
		users = web.select('users', where='active=1')
		
		# Defaultvalues for the form
		yearvalue = time.strftime("%Y")
		monthvalue = time.strftime("%m")
		dayvalue = time.strftime("%d")
		titlevalue = ''
		taskvalue = ''
		whovalue = 0

		# Template		
		headline = 'Add a TODO...'
		action = '/todo/add'

		todo = None
		message = None
		web.render('todoform.html')


	def POST(self):
		"""This method saves the data in the database."""

		# Authentification
		if todolib.valid_cookie() == False:
			user.validate().GET(redirect=web.ctx.path, message='Please log in...')
			return


        	i = web.input()

		deadline = i.year + "-" + i.month + "-" + i.day

		if i.title == '':
			i.title = i.task[0:20] + ' [...]'

		# Inserting the data
	       	todoid = web.insert('todos', title=i.title, task=i.task, deadline=deadline, who=i.who)
		
		# Sending a mail with the content of the new todo
		if config.newtodomail == True:
			todolib.sendNewTodoMail(todolib.getSession()['userid'], todoid)
		

		list().GET('TODO "' + i.title + '" added.') 	


class delete:
	def GET(self, name):
		# Authentification
		if todolib.valid_cookie() == False:
			user.validate().GET(redirect=web.ctx.path, message='Please log in...')
			return

		try:
			int(name)
		except ValueError:
			todotool.error().GET(message='Page does not exist.', description='Do not edit the URL on your own.')
			return
	
		query = web.update('todos', where='id='+name, deleted=1)
		
		if query == 0:
			todotool.error().GET(message='Page does not exist.', description='Do not edit the URL on your own.')
			return

		list().GET('TODO with id '+ name +' deleted.')



class show:
	def GET(self, name):
		"""This method shows a TODO."""

		# Authentification
		if todolib.valid_cookie() == False:
			user.validate().GET(redirect=web.ctx.path, message='Please log in...')
			return

		try:
			int(name)
		except ValueError:
			todotool.error().GET(message='Page does not exist.', description='Do not edit the URL on your own.')
			return


		todo = web.select('todos', where='id='+name, limit=1)

		try:
			todo = todo[0]
		except IndexError:
			todotool.error().GET(message='Page does not exist.', description='Do not edit the URL on your own.')
			return

		title = web.websafe(todo['title'])

		#One row of the text is one element of the sequenz
		task = web.websafe(todo['task'])
		task = task.split('\n')
		
		deadline = web.websafe(todo['deadline'])
		done = web.websafe(todo['done'])
		whoid = web.websafe(todo['who'])

		comment = web.websafe(todo['comment'])

		query = web.select('users', where='id='+ str(todo['who']), limit=1)
		query = query[0]


		username = web.websafe(query['name'])
		usermail = web.websafe(query['mail'])
		
		action = '/todo/edit/' + name
		message = None
		web.render('todoshow.html')



class edit:
	def GET(self, name):

		# Authentification
		if todolib.valid_cookie() == False:
			user.validate().GET(redirect=web.ctx.path, message='Please log in...')
			return
		
		try:
			int(name)
		except ValueError:
			todotool.error().GET(message='Page does not exist.', description='Do not edit the URL on your own.')
			return


		todo = web.select('todos', where='id='+name, limit=1)
		users = web.select('users')

		try:
			todo = todo[0]
		except IndexError:
			todotool.error().GET(message='Page does not exist.', description='Do not edit the URL on your own.')
			return
			

		titlevalue = web.websafe(todo['title'])
		taskvalue = web.websafe(todo['task'])
		whovalue = web.websafe(todo['who'])

		deadline = web.websafe(todo['deadline'])
		yearvalue = deadline[0:4]
		monthvalue = deadline[5:7]
		dayvalue = deadline[8:10]

		headline = 'Edit a TODO...'
		action = web.ctx.path
		message = None
		web.render('todoform.html')


	def POST(self, name):
		"""This method saves the data in the database."""

		# Authentification
		if todolib.valid_cookie() == False:
			user.validate().GET(redirect=web.ctx.path, message='Please log in...')
			return

		try:
			int(name)
		except ValueError:
			todotool.error().GET(message='Page does not exist.', description='Do not edit the URL on your own.')
			return


        	i = web.input()

		deadline = i.year + "-" + i.month + "-" + i.day

		if (i.done == 'on') or (i.done == '1'):
			done = 1
		else:
			done = 0
		
		# Updating the data
  	    	query = web.update('todos', where="id="+ i.id, title=i.title, task=i.task, deadline=deadline, who=i.who, done=done, comment=i.comment)
		
		if query == 0:
			todotool.error().GET(message='Page does not exist.', description='Do not edit the URL on your own.')
			return

		list().GET(message='Changes saved.')
	
